import { system, world, Vector } from '@minecraft/server'
const scoreboard = world.scoreboard;
const getData = scoreboard.getObjective.bind(scoreboard);
const addData = scoreboard.addObjective.bind(scoreboard);
const removeData = scoreboard.removeObjective.bind(scoreboard);

system.runInterval(() => {
	const players = world.getDimension("overworld").getPlayers();
	if (getData("daybreaks:enabled")) {
		for (let player of players) {
			let rayPlayer = player.dimension.getBlockFromRay(player.location, new Vector(0, 255, 0))?.block;
			// detection
			if (!player.hasTag("frame_immunity") && !rayPlayer && (world.getTimeOfDay() <= 12000)) {
				player.addTag("melted")
			}
			// melted tag
			if (player.hasTag("melted")) {
				player.getComponent("minecraft:movement").setCurrentValue(0.035)
			}
			if (player.hasTag("melted") && !player.hasTag("received")) {
				player.runCommandAsync("effect @s nausea 14 0")
				player.onScreenDisplay.setActionBar("§oYou feel strange...")
				player.addTag("received")
				
				// death
				system.runTimeout(() => {
					player.kill()
				}, 20*12)
			}
			// remove frame immunity
			if (player.getComponent("minecraft:health").currentValue != 0) {
				player.removeTag("frame_immunity")
			}
		}
	}
})
world.afterEvents.entityDie.subscribe((data) => {
	const deadEntity = data.deadEntity;
	if (getData("daybreaks:enabled") && deadEntity.typeId == "minecraft:player" && deadEntity.hasTag("melted")) {
		deadEntity.getComponent("minecraft:movement").resetToDefaultValue()
		deadEntity.dimension.spawnEntity("daybreaks:melted", deadEntity.location)
		deadEntity.removeTag("melted")
		deadEntity.removeTag("received")
		deadEntity.addTag("frame_immunity")
	}
})

world.beforeEvents.chatSend.subscribe((data) => {
	const sender = data.sender;
	const message = data.message;
	if (sender.isOp()) {
		if (message.startsWith("!daybreaks")) {
			let syntax = message.slice("!daybreaks".length).split(' ');
			data.cancel = true
			if (syntax[1] == "true") {
				sender.sendMessage("When Day Breaks is now enabled")
				system.run(() => {
					addData("daybreaks:enabled", "Enabled")
				})
			}
			if (syntax[1] == "false") {
				sender.sendMessage("When Day Breaks is now disabled")
				system.run(() => {
					removeData("daybreaks:enabled")
				})
			}
			if (message === "!daybreaks") {
				sender.sendMessage("type \"!daybreaks true/false\" to toggle When Day Breaks")
			}
			if (syntax[1] != "true" && syntax[1] != "false" && message != "!daybreaks") {
				sender.sendMessage(`§cUnkown command: !daybreaks >>${syntax[1]}<<. is not a valid command`)
			}
		}
		if (message.startsWith("!")) {
			let syntax = message.slice("!".length).split(' ');
			data.cancel = true
			if (syntax[0] != "daybreaks") {
				sender.sendMessage(`§cUnkown command: ${syntax[0]}. is not a valid command`)
			}
		}
	}
})